#include <gtk/gtk.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include "defines.h"
#include "vpdefines.h"
#include "globals.h"
#include "globals_gtk.h"

#include "protos_volume.h"
#include "protos_gui.h"
#include "protos_menus.h"


/* -------------
   Gtk Callbacks
   ------------- */
gboolean on_darea_expose (GtkWidget *widget, GdkEventExpose *event, gpointer user_data){
   gint xorigin, yorigin;

   gdk_draw_rectangle(widget->window, widget->style->white_gc, TRUE, 0, 0,
                      widget->allocation.width, widget->allocation.height);

   if(image != NULL){
      xorigin = (DAREA_WIDTH - _VolumeDimensions.x * imageScaleX) / 2;
      yorigin = (DAREA_HEIGHT - _VolumeDimensions.y * imageScaleY) / 2;
      gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
                          xorigin, yorigin, _VolumeDimensions.x * imageScaleX, _VolumeDimensions.y * imageScaleY,
                          GDK_RGB_DITHER_MAX, (guchar*)image, _VolumeDimensions.x * 3 * imageScaleX);
   }

}




void setDpSensitivity(){
   int i;

   i = numDensity;
   numDensity = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(numDensityCombo)->entry)));
   if(numDensity > 18) numDensity = i;

   for(i = 0; i < (numDensity / 2); i++){
      gtk_widget_set_sensitive(GTK_WIDGET(dpx[i][0]), 1);
      gtk_widget_set_sensitive(GTK_WIDGET(dpy[i][0]), 1);
      gtk_widget_set_sensitive(GTK_WIDGET(dpx[i][1]), 1);
      gtk_widget_set_sensitive(GTK_WIDGET(dpy[i][1]), 1);
   }
   for(i = (numDensity / 2); i < 9; i++){
      gtk_widget_set_sensitive(GTK_WIDGET(dpx[i][0]), 0);
      gtk_widget_set_sensitive(GTK_WIDGET(dpy[i][0]), 0);
      gtk_widget_set_sensitive(GTK_WIDGET(dpx[i][1]), 0);
      gtk_widget_set_sensitive(GTK_WIDGET(dpy[i][1]), 0);
   }
   
}

void setGpSensitivity(){
   int i;

   i = numGradient;
   numGradient = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(numGradientCombo)->entry)));
   if(numGradient > 7) numGradient = i;
   
   for(i = 0; i < numGradient; i++){
      gtk_widget_set_sensitive(GTK_WIDGET(gpx[i]), 1);
      gtk_widget_set_sensitive(GTK_WIDGET(gpy[i]), 1);
   }
   for(i = numGradient; i < 7; i++){
      gtk_widget_set_sensitive(GTK_WIDGET(gpx[i]), 0);
      gtk_widget_set_sensitive(GTK_WIDGET(gpy[i]), 0);
   }

}

void setNumMaterials(){
   int i;

   i = numMaterials;
   numMaterials = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(numMaterialsCombo)->entry)));
   if(numMaterials > 6) numMaterials = i;

   for(i = 0; i < numMaterials; i++){
      gtk_widget_set_sensitive(GTK_WIDGET(weightEntry[i][0]), 1);
      gtk_widget_set_sensitive(GTK_WIDGET(weightEntry[i][1]), 1);
   }
   for(i = numMaterials; i < 6; i++){
      gtk_widget_set_sensitive(GTK_WIDGET(weightEntry[i][0]), 0);
      gtk_widget_set_sensitive(GTK_WIDGET(weightEntry[i][1]), 0);
   }
}



/* --------------------------
   Gtk GUI Building Functions
   -------------------------- */
void createMainWindow(){
   GtkWidget *window, *menubar, *hbox, *vbox, *vbox1, *vbox2;

   /* Create new top level window */
   window = gtk_window_new( GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(window), "GVS");
   gtk_container_border_width(GTK_CONTAINER(window), 2);

   /* Quit form main if got delete event */
   gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

   vbox = gtk_vbox_new(FALSE, 0);
   
   create_menu(window, &menubar);
   gtk_box_pack_start(GTK_BOX(vbox), menubar, TRUE, FALSE, 0);
   gtk_widget_show(menubar);

   hbox = gtk_hbox_new(FALSE, 10);
   vbox1 = gtk_vbox_new(FALSE, 10);
   vbox2 = gtk_vbox_new(FALSE, 10);

   populateVbox1(vbox1);
   populateVbox2(vbox2);

   gtk_box_pack_start(GTK_BOX(hbox), vbox1, TRUE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(vbox1));
   gtk_widget_show(GTK_WIDGET(vbox2));

   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);
   gtk_widget_show(hbox);

   gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(vbox));
   gtk_widget_show(GTK_WIDGET(vbox));
   gtk_widget_show(GTK_WIDGET(window));
}


void populateVbox1(GtkWidget *vbox){
   GtkWidget *hbox, *button, *table;

   darea = gtk_drawing_area_new();
   gtk_drawing_area_size(GTK_DRAWING_AREA(darea), DAREA_WIDTH, DAREA_HEIGHT);
   /*gdk_rgb_gc_set_background(darea->style->fg_gc[GTK_STATE_NORMAL], 0x000000);*/
   gtk_signal_connect (GTK_OBJECT (darea), "expose-event", GTK_SIGNAL_FUNC (on_darea_expose), NULL);
   
   gtk_box_pack_start(GTK_BOX(vbox), darea, TRUE, FALSE, 0);
   gtk_widget_show(darea);

   /* Add rotation buttons */
   table = gtk_table_new(2, 3, TRUE);
   
   button = gtk_button_new_with_label("");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rotateZ), NULL);
   gtk_widget_set_usize(GTK_WIDGET(button), 20, 20);
   gtk_table_attach(GTK_TABLE(table), button, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show(GTK_WIDGET(button));

   button = gtk_button_new_with_label("");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rotateMinusZ), NULL);
   gtk_widget_set_usize(GTK_WIDGET(button), 20, 20);
   gtk_table_attach(GTK_TABLE(table), button, 2, 3, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show(GTK_WIDGET(button));

   button = gtk_button_new_with_label("");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rotateMinusY), NULL);
   gtk_widget_set_usize(GTK_WIDGET(button), 20, 20);
   gtk_table_attach(GTK_TABLE(table), button, 0, 1, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show(GTK_WIDGET(button));

   button = gtk_button_new_with_label("");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rotateY), NULL);
   gtk_widget_set_usize(GTK_WIDGET(button), 20, 20);
   gtk_table_attach(GTK_TABLE(table), button, 2, 3, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show(GTK_WIDGET(button));

   button = gtk_button_new_with_label("");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rotateX), NULL);
   gtk_widget_set_usize(GTK_WIDGET(button), 20, 20);
   gtk_table_attach(GTK_TABLE(table), button, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show(GTK_WIDGET(button));

   button = gtk_button_new_with_label("");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rotateMinusX), NULL);
   gtk_widget_set_usize(GTK_WIDGET(button), 20, 20);
   gtk_table_attach(GTK_TABLE(table), button, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_widget_show(GTK_WIDGET(button));

   gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, FALSE, 0);
   gtk_widget_show(table);

   /* Add render button */
   hbox = gtk_hbox_new(FALSE, 5);
   button = gtk_button_new_with_label("Render");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(render_volume), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(button));

   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);
   gtk_widget_show(hbox);
}

void populateVbox2(GtkWidget *vbox){
   GtkWidget *notebook;

   notebook = create_notebook();
   gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0); 
   gtk_widget_show(notebook);
}

GtkWidget* create_notebook(){
   GtkWidget *notebook, *label, *pageFrame;

   notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK (notebook), GTK_POS_TOP);

   label = gtk_label_new("Classification");
   pageFrame = gtk_frame_new("Classification");
   populateClassifyPage(pageFrame);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("Materials");
   pageFrame = gtk_frame_new("Materials");
   populateMaterialsPage(pageFrame);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("Lights");
   pageFrame = gtk_frame_new("Lights");
   populateLightsPage(pageFrame);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);
   
   label = gtk_label_new("Shading");
   pageFrame = gtk_frame_new("Shading");
   populateShadingPage(pageFrame);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   return notebook;
}

void populateClassifyPage(GtkWidget *page){
   GtkWidget *hbox, *vbox, *label, *button;
   GList *dpItemList = NULL;
   GList *gpItemList = NULL;
   int i;

   dpItemList = g_list_append(dpItemList, "2");
   dpItemList = g_list_append(dpItemList, "4");
   dpItemList = g_list_append(dpItemList, "6");
   dpItemList = g_list_append(dpItemList, "8"); 
   dpItemList = g_list_append(dpItemList, "10");
   dpItemList = g_list_append(dpItemList, "12"); 
   dpItemList = g_list_append(dpItemList, "14"); 
   dpItemList = g_list_append(dpItemList, "16");
   dpItemList = g_list_append(dpItemList, "18");

   gpItemList = g_list_append(gpItemList, "2");
   gpItemList = g_list_append(gpItemList, "3");
   gpItemList = g_list_append(gpItemList, "4"); 
   gpItemList = g_list_append(gpItemList, "5");
   gpItemList = g_list_append(gpItemList, "6"); 
   gpItemList = g_list_append(gpItemList, "7");

   vbox = gtk_vbox_new(FALSE, 5);

   /* ---- Number of Density Points Combo Box ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("Number of Density Points");
   numDensityCombo = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(numDensityCombo), dpItemList);
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(numDensityCombo)->entry), "changed", GTK_SIGNAL_FUNC(setDpSensitivity), NULL);

   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), numDensityCombo, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(label));
   gtk_widget_show(GTK_WIDGET(numDensityCombo));

   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(hbox));

   /* ---- Density Point Entry Fields ---- */
   for(i = 0; i < 9; i++){
      hbox = gtk_hbox_new(FALSE, 5);
      
      dpx[i][0] = gtk_entry_new();
      gtk_entry_set_max_length(GTK_ENTRY(dpx[i][0]), 5);
      gtk_widget_set_usize(dpx[i][0], 40, 22);
      gtk_box_pack_start(GTK_BOX(hbox), dpx[i][0], FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(dpx[i][0]));

      dpx[i][1] = gtk_entry_new();
      gtk_entry_set_max_length(GTK_ENTRY(dpx[i][1]), 5);
      gtk_widget_set_usize(dpx[i][1], 40, 22);
      gtk_box_pack_start(GTK_BOX(hbox), dpx[i][1], FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(dpx[i][1]));

      dpy[i][0] = gtk_entry_new();
      gtk_entry_set_max_length(GTK_ENTRY(dpy[i][0]), 5);
      gtk_widget_set_usize(dpy[i][0], 40, 22);
      gtk_box_pack_start(GTK_BOX(hbox), dpy[i][0], FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(dpy[i][0]));

      dpy[i][1] = gtk_entry_new();
      gtk_entry_set_max_length(GTK_ENTRY(dpy[i][1]), 5);
      gtk_widget_set_usize(dpy[i][1], 40, 22);
      gtk_box_pack_start(GTK_BOX(hbox), dpy[i][1], FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(dpy[i][1]));

      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(hbox));
   }
   
   /* ---- Number of Gradient Points Combo Box ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("Number of Gradient Points");
   numGradientCombo = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(numGradientCombo), gpItemList);
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(numGradientCombo)->entry), "changed", GTK_SIGNAL_FUNC(setGpSensitivity), NULL);

   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), numGradientCombo, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(label));
   gtk_widget_show(GTK_WIDGET(numGradientCombo));

   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(hbox));
   
   /* ---- Gradient Point Entry Fields ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   for(i = 0; i < 7; i++){
      gpx[i] =gtk_entry_new();
      gtk_entry_set_max_length(GTK_ENTRY(gpx[i]), 5);
      gtk_widget_set_usize(gpx[i], 40, 22);
      gtk_box_pack_start(GTK_BOX(hbox), gpx[i], FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(gpx[i]));
   }
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(hbox));

   hbox = gtk_hbox_new(FALSE, 5);
   for(i = 0; i < 7; i++){
      gpy[i] =gtk_entry_new();
      gtk_entry_set_max_length(GTK_ENTRY(gpy[i]), 5);
      gtk_widget_set_usize(gpy[i], 40, 22);
      gtk_box_pack_start(GTK_BOX(hbox), gpy[i], FALSE, FALSE, 0);
      gtk_widget_show(GTK_WIDGET(gpy[i]));
   }
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(hbox));

   /* ---- Apply button ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   button = gtk_button_new_with_label("Apply");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(classify_volume), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(button));
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(hbox));

   gtk_container_add(GTK_CONTAINER(page), vbox);
   gtk_widget_show(vbox);
}

void populateMaterialsPage(GtkWidget *page){
   GtkWidget *notebook; 
   
   notebook = create_materials_notebook();
   gtk_container_add(GTK_CONTAINER(page), notebook);
   gtk_widget_show(notebook);
}

void populateShadingPage(GtkWidget *page){
   GtkWidget *button, *hbox, *vbox, *label;
   GList *nmItemList = NULL;
   
   nmItemList = g_list_append(nmItemList, "1");
   nmItemList = g_list_append(nmItemList, "2");
   nmItemList = g_list_append(nmItemList, "3"); 
   nmItemList = g_list_append(nmItemList, "4");
   nmItemList = g_list_append(nmItemList, "5"); 
   nmItemList = g_list_append(nmItemList, "6");
   
   vbox = gtk_vbox_new(FALSE, 5);
   
   /* ---- Number of Materials Combo Box ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("Number of Materials");
   numMaterialsCombo = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(numMaterialsCombo), nmItemList);
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(numMaterialsCombo)->entry), "changed", GTK_SIGNAL_FUNC(setNumMaterials), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), numMaterialsCombo, FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(numMaterialsCombo);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   /* ---- Number of Lights Combo Box ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("Number of Lights");
   numLightsCombo = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(numLightsCombo), nmItemList);
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(numLightsCombo)->entry), "changed", GTK_SIGNAL_FUNC(setNumLights), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), numLightsCombo, FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(numLightsCombo);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   /* ---- Weight Table ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("Weight Table Settings");
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("1");
   weightEntry[0][0] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[0][0]), 5);
   gtk_widget_set_usize(weightEntry[0][0], 40, 22);
   weightEntry[0][1] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[0][1]), 5);
   gtk_widget_set_usize(weightEntry[0][1], 40, 22);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[0][0], FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[0][1], FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(weightEntry[0][0]);
   gtk_widget_show(weightEntry[0][1]);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("2");
   weightEntry[1][0] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[1][0]), 5);
   gtk_widget_set_usize(weightEntry[1][0], 40, 22);
   weightEntry[1][1] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[1][1]), 5);
   gtk_widget_set_usize(weightEntry[1][1], 40, 22);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[1][0], FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[1][1], FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(weightEntry[1][0]);
   gtk_widget_show(weightEntry[1][1]);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("3");
   weightEntry[2][0] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[2][0]), 5);
   gtk_widget_set_usize(weightEntry[2][0], 40, 22);
   weightEntry[2][1] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[2][1]), 5);
   gtk_widget_set_usize(weightEntry[2][1], 40, 22);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[2][0], FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[2][1], FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(weightEntry[2][0]);
   gtk_widget_show(weightEntry[2][1]);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("4");
   weightEntry[3][0] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[3][0]), 5);
   gtk_widget_set_usize(weightEntry[3][0], 40, 22);
   weightEntry[3][1] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[3][1]), 5);
   gtk_widget_set_usize(weightEntry[3][1], 40, 22);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[3][0], FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[3][1], FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(weightEntry[3][0]);
   gtk_widget_show(weightEntry[3][1]);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("5");
   weightEntry[4][0] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[4][0]), 5);
   gtk_widget_set_usize(weightEntry[4][0], 40, 22);
   weightEntry[4][1] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[4][1]), 5);
   gtk_widget_set_usize(weightEntry[4][1], 40, 22);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[4][0], FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[4][1], FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(weightEntry[4][0]);
   gtk_widget_show(weightEntry[4][1]);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   hbox = gtk_hbox_new(FALSE, 5);
   label = gtk_label_new("6");
   weightEntry[5][0] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[5][0]), 5);
   gtk_widget_set_usize(weightEntry[5][0], 40, 22);
   weightEntry[5][1] = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(weightEntry[5][1]), 5);
   gtk_widget_set_usize(weightEntry[5][1], 40, 22);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[5][0], FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), weightEntry[5][1], FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(weightEntry[5][0]);
   gtk_widget_show(weightEntry[5][1]);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   /* ---- Apply button ---- */
   hbox = gtk_hbox_new(FALSE, 5);
   button = gtk_button_new_with_label("Apply");
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(shade_volume), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(button));
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(GTK_WIDGET(hbox));

   gtk_container_add(GTK_CONTAINER(page), vbox);
   gtk_widget_show(vbox);
}

void populateLightsPage(GtkWidget *page){
   GtkWidget *notebook; 
   
   notebook = create_lights_notebook();
   gtk_container_add(GTK_CONTAINER(page), notebook);
   gtk_widget_show(notebook);   
}

GtkWidget* create_materials_notebook(){
   GtkWidget *notebook, *label, *pageFrame;

   notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK (notebook), GTK_POS_LEFT);

   label = gtk_label_new("1");
   pageFrame = gtk_frame_new("Material 1");
   populateMatPage(pageFrame, 1);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("2");
   pageFrame = gtk_frame_new("Material 2");
   populateMatPage(pageFrame, 2);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("3");
   pageFrame = gtk_frame_new("Material 3");
   populateMatPage(pageFrame, 3);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("4");
   pageFrame = gtk_frame_new("Material 4");
   populateMatPage(pageFrame, 4);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("5");
   pageFrame = gtk_frame_new("Material 5");
   populateMatPage(pageFrame, 5);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("6");
   pageFrame = gtk_frame_new("Material 6");
   populateMatPage(pageFrame, 6);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   return notebook;
}

void populateMatPage(GtkWidget *page, int pagenum){
   GtkWidget *vbox, *hbox, *label;
   GtkAdjustment *adj;
   int pageindex;

   pageindex = pagenum - 1;
   vbox = gtk_vbox_new(FALSE, 5);

   /* Ambient sliders */
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   label = gtk_label_new("Ambient Reflectance");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matAmbScale[pageindex][0] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matAmbScale[pageindex][0]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matAmbScale[pageindex][0]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matAmbScale[pageindex][0], FALSE, FALSE, 0);
   gtk_widget_show(matAmbScale[pageindex][0]);

   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matAmbScale[pageindex][1] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matAmbScale[pageindex][1]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matAmbScale[pageindex][1]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matAmbScale[pageindex][1], FALSE, FALSE, 0);
   gtk_widget_show(matAmbScale[pageindex][1]);
   
   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matAmbScale[pageindex][2] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matAmbScale[pageindex][2]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matAmbScale[pageindex][2]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matAmbScale[pageindex][2], FALSE, FALSE, 0);
   gtk_widget_show(matAmbScale[pageindex][2]);
   
   /* Diffuse sliders */
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   label = gtk_label_new("Diffuse Reflectance");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matDiffScale[pageindex][0] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matDiffScale[pageindex][0]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matDiffScale[pageindex][0]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matDiffScale[pageindex][0], FALSE, FALSE, 0);
   gtk_widget_show(matDiffScale[pageindex][0]);

   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matDiffScale[pageindex][1] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matDiffScale[pageindex][1]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matDiffScale[pageindex][1]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matDiffScale[pageindex][1], FALSE, FALSE, 0);
   gtk_widget_show(matDiffScale[pageindex][1]);
   
   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matDiffScale[pageindex][2] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matDiffScale[pageindex][2]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matDiffScale[pageindex][2]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matDiffScale[pageindex][2], FALSE, FALSE, 0);
   gtk_widget_show(matDiffScale[pageindex][2]);

   /* Specular sliders */
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   label = gtk_label_new("Specular Reflectance");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matSpecScale[pageindex][0] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matSpecScale[pageindex][0]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matSpecScale[pageindex][0]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matSpecScale[pageindex][0], FALSE, FALSE, 0);
   gtk_widget_show(matSpecScale[pageindex][0]);

   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matSpecScale[pageindex][1] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matSpecScale[pageindex][1]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matSpecScale[pageindex][1]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matSpecScale[pageindex][1], FALSE, FALSE, 0);
   gtk_widget_show(matSpecScale[pageindex][1]);
   
   adj = gtk_adjustment_new(0.25, 0.0, 1.0, 0.01, 0.1, 0.0);
   matSpecScale[pageindex][2] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matSpecScale[pageindex][2]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matSpecScale[pageindex][2]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matSpecScale[pageindex][2], FALSE, FALSE, 0);
   gtk_widget_show(matSpecScale[pageindex][2]);

   /* Shinyness sliders */
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   label = gtk_label_new("Shinyness");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   adj = gtk_adjustment_new(10.0, 0.0, 25.0, 0.5, 5.0, 0.0);
   matShinyScale[pageindex] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(matShinyScale[pageindex]), 2);
   gtk_range_set_update_policy(GTK_RANGE(matShinyScale[pageindex]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), matShinyScale[pageindex], FALSE, FALSE, 0);
   gtk_widget_show(matShinyScale[pageindex]);

   gtk_container_add(GTK_CONTAINER(page), vbox);
   gtk_widget_show(vbox);
}

GtkWidget* create_lights_notebook(){
   GtkWidget *notebook, *label, *pageFrame;

   notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK (notebook), GTK_POS_LEFT);

   label = gtk_label_new("1");
   pageFrame = gtk_frame_new("Light 1");
   populateLightPage(pageFrame, 1);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("2");
   pageFrame = gtk_frame_new("Light 2");
   populateLightPage(pageFrame, 2);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("3");
   pageFrame = gtk_frame_new("Light 3");
   populateLightPage(pageFrame, 3);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("4");
   pageFrame = gtk_frame_new("Light 4");
   populateLightPage(pageFrame, 4);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("5");
   pageFrame = gtk_frame_new("Light 5");
   populateLightPage(pageFrame, 5);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   label = gtk_label_new("6");
   pageFrame = gtk_frame_new("Light 6");
   populateLightPage(pageFrame, 6);
   gtk_notebook_append_page(GTK_NOTEBOOK (notebook), pageFrame, label);
   gtk_widget_show(pageFrame);

   return notebook;
}

void populateLightPage(GtkWidget *page, int pagenum){
   GtkWidget *vbox, *hbox, *label;
   GtkAdjustment *adj;
   int pageindex;

   pageindex = pagenum - 1;
   vbox = gtk_vbox_new(FALSE, 5);

   /* Directions sliders */
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   label = gtk_label_new("Light Direction Vector");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   adj = gtk_adjustment_new(0.3, -1.0, 1.0, 0.01, 0.1, 0.0);
   lightDirScale[pageindex][0] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(lightDirScale[pageindex][0]), 2);
   gtk_range_set_update_policy(GTK_RANGE(lightDirScale[pageindex][0]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), lightDirScale[pageindex][0], FALSE, FALSE, 0);
   gtk_widget_show(lightDirScale[pageindex][0]);

   adj = gtk_adjustment_new(0.3, -1.0, 1.0, 0.01, 0.1, 0.0);
   lightDirScale[pageindex][1] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(lightDirScale[pageindex][1]), 2);
   gtk_range_set_update_policy(GTK_RANGE(lightDirScale[pageindex][1]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), lightDirScale[pageindex][1], FALSE, FALSE, 0);
   gtk_widget_show(lightDirScale[pageindex][1]);
   
   adj = gtk_adjustment_new(0.3, -1.0, 1.0, 0.01, 0.1, 0.0);
   lightDirScale[pageindex][2] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(lightDirScale[pageindex][2]), 2);
   gtk_range_set_update_policy(GTK_RANGE(lightDirScale[pageindex][2]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), lightDirScale[pageindex][2], FALSE, FALSE, 0);
   gtk_widget_show(lightDirScale[pageindex][2]);

   /* Color sliders */
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);

   label = gtk_label_new("Light RGB Values");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);

   adj = gtk_adjustment_new(1.0, 0.0, 1.0, 0.01, 0.1, 0.0);
   lightColorScale[pageindex][0] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(lightColorScale[pageindex][0]), 2);
   gtk_range_set_update_policy(GTK_RANGE(lightColorScale[pageindex][0]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), lightColorScale[pageindex][0], FALSE, FALSE, 0);
   gtk_widget_show(lightColorScale[pageindex][0]);

   adj = gtk_adjustment_new(1.0, 0.0, 1.0, 0.01, 0.1, 0.0);
   lightColorScale[pageindex][1] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(lightColorScale[pageindex][1]), 2);
   gtk_range_set_update_policy(GTK_RANGE(lightColorScale[pageindex][1]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), lightColorScale[pageindex][1], FALSE, FALSE, 0);
   gtk_widget_show(lightColorScale[pageindex][1]);
   
   adj = gtk_adjustment_new(1.0, 0.0, 1.0, 0.01, 0.1, 0.0);
   lightColorScale[pageindex][2] = gtk_hscale_new(adj);
   gtk_scale_set_digits(GTK_SCALE(lightColorScale[pageindex][2]), 2);
   gtk_range_set_update_policy(GTK_RANGE(lightColorScale[pageindex][2]), GTK_UPDATE_DISCONTINUOUS);
   gtk_box_pack_start(GTK_BOX(vbox), lightColorScale[pageindex][2], FALSE, FALSE, 0);
   gtk_widget_show(lightColorScale[pageindex][2]);


   gtk_container_add(GTK_CONTAINER(page), vbox);
   gtk_widget_show(vbox);

}

void setNumLights(){

   numLights = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(numLightsCombo)->entry)));
   
}

