#include <gtk/gtk.h>
#include <stdlib.h>
#include "defines.h"
#include "vpdefines.h"
#include "globals.h"
#include "globals_gtk.h"

#include "protos_volume.h"
#include "protos_gui.h"


/* ---- Function Prototypes ---- */
void init_globals();
void init_gui();



int main(int argc, char **argv){

   init_globals();

   if(argc == 2) {
      /* read and load the volume input file */
      if(readVolume(argv[1]) != 1) exit(-1);
      load_volume();
   }
   
   gtk_init(&argc, &argv);
   gdk_rgb_init();
   gtk_widget_set_default_colormap (gdk_rgb_get_cmap());
   gtk_widget_set_default_visual (gdk_rgb_get_visual());
   
   createMainWindow();
   init_gui();

   gtk_main();

   return 0;
}

void init_globals(){
   int i;
   
   _rawdata = NULL;
   vpc = NULL;
   image = NULL;
   shade_table = NULL;
   weight_table = NULL;

   numDensity = 2;
   numGradient = 4;
   numMaterials = 1;
   numLights = 1;

   viewVolume[0][0] = -0.5;
   viewVolume[0][1] = 0.5;
   viewVolume[1][0] = -0.5;
   viewVolume[1][1] = 0.5;
   viewVolume[2][0] = -0.5;
   viewVolume[2][1] = 0.5;

   imageScaleX = 1.0;
   imageScaleY = 1.0;

   DensityRampX[0] = 0;
   DensityRampY[0] = 0.0;
   for(i = 1; i < 18; i++){
      DensityRampX[i] = 255;
      DensityRampY[i] = 1.0;
   }


   GradientRampX[0] = 0;
   GradientRampX[1] = 5;
   GradientRampX[2] = 20;
   GradientRampX[3] = 221;
   GradientRampX[4] = 255;
   GradientRampX[5] = 255;
   GradientRampX[6] = 255;
   
   GradientRampY[0] = 0.0;
   GradientRampY[1] = 0.0;
   GradientRampY[2] = 1.0;
   GradientRampY[3] = 1.0;
   GradientRampY[4] = 1.0;
   GradientRampY[5] = 1.0;
   GradientRampY[6] = 1.0;
}

void init_gui(){
   GtkAdjustment *adj;

   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(numDensityCombo)->entry), "2");
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(numGradientCombo)->entry), "4");

   gtk_entry_set_text(GTK_ENTRY(gpx[0]), "0");
   gtk_entry_set_text(GTK_ENTRY(gpx[1]), "5");
   gtk_entry_set_text(GTK_ENTRY(gpx[2]), "20");
   gtk_entry_set_text(GTK_ENTRY(gpx[3]), "221");
   gtk_entry_set_text(GTK_ENTRY(gpy[0]), "0.0");
   gtk_entry_set_text(GTK_ENTRY(gpy[1]), "0.0");
   gtk_entry_set_text(GTK_ENTRY(gpy[2]), "1.0");
   gtk_entry_set_text(GTK_ENTRY(gpy[3]), "1.0");

   setDpSensitivity();
   setGpSensitivity();
   setNumMaterials();

   adj = gtk_range_get_adjustment(GTK_RANGE(matAmbScale[0][0]));
   gtk_adjustment_set_value(adj, 0.18);
   adj = gtk_range_get_adjustment(GTK_RANGE(matAmbScale[0][1]));
   gtk_adjustment_set_value(adj, 0.18);
   adj = gtk_range_get_adjustment(GTK_RANGE(matAmbScale[0][2]));
   gtk_adjustment_set_value(adj, 0.18);

   adj = gtk_range_get_adjustment(GTK_RANGE(matDiffScale[0][0]));
   gtk_adjustment_set_value(adj, 0.35);
   adj = gtk_range_get_adjustment(GTK_RANGE(matDiffScale[0][1]));
   gtk_adjustment_set_value(adj, 0.35);
   adj = gtk_range_get_adjustment(GTK_RANGE(matDiffScale[0][2]));
   gtk_adjustment_set_value(adj, 0.35);

   adj = gtk_range_get_adjustment(GTK_RANGE(matSpecScale[0][0]));
   gtk_adjustment_set_value(adj, 0.39);
   adj = gtk_range_get_adjustment(GTK_RANGE(matSpecScale[0][1]));
   gtk_adjustment_set_value(adj, 0.39);
   adj = gtk_range_get_adjustment(GTK_RANGE(matSpecScale[0][2]));
   gtk_adjustment_set_value(adj, 0.39);

   adj = gtk_range_get_adjustment(GTK_RANGE(matShinyScale[0]));
   gtk_adjustment_set_value(adj, 10.0);
}
