#include <fcntl.h>
#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <stdlib.h>
#include "defines.h"
#include "vpdefines.h"

#define LENDIAN


/*+-------------------------------------------------------------------*/
/*+---------------+
  |   SwapBytes   |
  +---------------+*/ 
#ifdef  LENDIAN
void	SwapBytes(
   int	*intPtr,    /* pointer of words requiring byte swapping */
   unsigned int	sizeBytes   /* number of bytes in array */
){
   int				sizeInts, i;
   unsigned char	*bytePtr;

   sizeInts = sizeBytes / 4;
   for (i = 0; i < sizeInts; i++)
   {
      bytePtr = (unsigned char *) intPtr;
      *intPtr++ = ((bytePtr[0]<<24) | (bytePtr[1]<<16) | (bytePtr[2]<<8) | bytePtr[3]);
   }
}
#endif

unsigned short doubleToShort(double d){
   float inc;

   inc = (MAX_ED - MIN_ED) / 65536;

   return d / inc;
}

unsigned char doubleToByte(double d){
   float inc;

   inc = (MAX_ED - MIN_ED) / 255;

   return d / inc;
}

int readVol(int input_fd, char *filename){
   int     l;
   int     totalDepth;
   int     bytesPerPixel;
   int     pixelsPerSlice;
   int     pixelsPerVolume;
   int     bytesPerSlice;

   enum	  errorCode;
   
   read(input_fd, (void *) &_HeaderLength, sizeof(int));
   read(input_fd, (void *) &_VolumeDimensions, sizeof(C_IntTuple));
   read(input_fd, (void *) &_PixelDepth, sizeof(int));
   read(input_fd, (void *) &_IndexBits, sizeof(int));
#ifdef  LENDIAN
   SwapBytes((int *) &_HeaderLength, sizeof(int));
   SwapBytes((int *) &_VolumeDimensions, sizeof(C_IntTuple));
   SwapBytes((int *) &_PixelDepth,sizeof(int));
   SwapBytes((int *) &_IndexBits, sizeof(int));
#endif
   _PixelDepth -= _IndexBits;
   if (_MagicNumber == C_VR_MAGIC_NUMBER){
      read( input_fd, (void *) &_ScaleX, sizeof(float));
      read( input_fd, (void *) &_ScaleY, sizeof(float));
      read( input_fd, (void *) &_ScaleZ, sizeof(float));
      read( input_fd, (void *) &_RotX, sizeof(float));
      read( input_fd, (void *) &_RotY, sizeof(float));
      read( input_fd, (void *) &_RotZ, sizeof(float));
#ifdef  LENDIAN
      SwapBytes ((int *) &_ScaleX, sizeof(float));
      SwapBytes ((int *) &_ScaleY, sizeof(float));
      SwapBytes ((int *) &_ScaleZ, sizeof(float));
      SwapBytes ((int *) &_RotX, sizeof(float));
      SwapBytes ((int *) &_RotY, sizeof(float));
      SwapBytes ((int *) &_RotZ, sizeof(float));
#endif
   }
   else{
      _ScaleX = (float) 1.0;
      _ScaleY = (float) 1.0;
      _ScaleZ = (float) 1.0;
      _RotX = (float) 0.0;
      _RotY = (float) 0.0;
      _RotZ = (float) 0.0;
   }


/*----- move the file pointer to the first voxel -----*/

   lseek(input_fd, _HeaderLength, SEEK_SET);        


/*----- first, let's figure out the pixel format -----*/

   totalDepth = _PixelDepth + _IndexBits;

   switch (totalDepth){
      case 8:
         _PixelFormat = C_INTENSITY8;
         _PixelType = C_UNSIGNED_BYTE;
         break;

      case 16:
         if (_IndexBits == 0){
            _PixelFormat = C_INTENSITY16;
            _PixelType = C_UNSIGNED_SHORT;
         }
         else if (_IndexBits == 4){
            _PixelFormat = C_INDEX4_INTENSITY12;
            _PixelType = C_UNSIGNED_SHORT;
         }
         else if (_IndexBits == 8){
            _PixelFormat = C_INDEX8_INTENSITY8;
            _PixelType = C_UNSIGNED_SHORT;
         }
         else{
            printf (
                   "This combination of index (%d bits) & pixel (%d bits)\n",
                   _IndexBits, _PixelDepth);
            printf ("depths is not supported\n");
            return (-1);
         }
         break;

      default:
         printf ("This combination of index (%d bits) & pixel (%d bits)\n",
                _IndexBits, _PixelDepth);
         printf ("depths is not supported\n");
         return (-1);
         break;
   }


/*----- determine the size of the dataset -----*/

   if(_PixelType == C_UNSIGNED_BYTE)
      bytesPerPixel = sizeof (unsigned char);
   else 
      bytesPerPixel = sizeof (unsigned short) / 2;

   pixelsPerSlice = _VolumeDimensions.x * _VolumeDimensions.y;
   pixelsPerVolume = pixelsPerSlice * _VolumeDimensions.z;
   bytesPerSlice = pixelsPerSlice * bytesPerPixel;


/*----- create buffer big enough for the volume -----*/

   _rawdata = (unsigned char *) calloc (pixelsPerVolume, bytesPerPixel);
   if(_rawdata == (unsigned char *) NULL){
      printf ("An error occurred allocating a volume buffer\n");
      exit (-1);
   }


/*----- read slices from the file and add to the volume -----*/

   printf ("bytesPerPixel = %d\n", bytesPerPixel);
   printf ("pixelsPerVolume = %d\n", pixelsPerVolume);

   if ((l = read(input_fd, _rawdata, (bytesPerPixel * pixelsPerVolume)))
      != (bytesPerPixel * pixelsPerVolume))
   {
      printf ("Reading volume failed\n");
      return (-1);
   }


/*----- close file and free buffer -----*/

   close(input_fd);


/*----- print information about the loaded dataset -----*/

   printf ("\nFile: [ %s ]\n", filename);
   printf ("Image Width  = %d pixels\n", _VolumeDimensions.x);
   printf ("Image Height = %d pixels\n", _VolumeDimensions.y);
   printf ("Image Frames = %d pixels\n", _VolumeDimensions.z);
   printf ("Pixel depth  = %d bits\n", _PixelDepth);
   switch (_PixelFormat){
      case  C_INTENSITY8:
         printf ("Pixel format = C_INTENSITY8\n");
         break;

      case  C_INTENSITY16:
         printf ("Pixel format = C_INTENSITY16\n");
         break;

      case  C_INDEX4_INTENSITY12:
         printf ("Pixel format = C_INDEX4_INTENSITY12\n");
         break;

      case  C_INDEX8_INTENSITY8:
         printf ("Pixel format = C_INDEX8_INTENSITY8\n");
         break;
   }

   if(_PixelType == C_UNSIGNED_BYTE )
      printf ("Image type  = C_UNSIGNED_BYTE\n");
   else
      printf ("Image type  = C_UNSIGNED_SHORT\n");

   if(_Manufacturer[0])
      printf ("Original manufacturer is %s\n", _Manufacturer);
   if(_OriginalFileName[0])
      printf ("Original file name is %s\n", _OriginalFileName);
   
   return 1;
}

int readDen(int input_fd, char *filename){
   
   return 1;
}

int readED(int input_fd, char *filename){
   int     l; 
   int     bytesPerPixel;
   int     pixelsPerSlice;
   int     pixelsPerVolume;
   int     bytesPerSlice;
   double  *tempdata;
   unsigned char *tempdata2;

   read(input_fd, (void *) &_VolumeDimensions, sizeof(C_IntTuple));
   
   _PixelFormat = C_INTENSITY8;
   _PixelType = C_UNSIGNED_BYTE;
   _PixelDepth = 8;

   /*----- determine the size of the dataset -----*/
   bytesPerPixel = sizeof(double);
   pixelsPerSlice = _VolumeDimensions.x * _VolumeDimensions.y;
   pixelsPerVolume = pixelsPerSlice * _VolumeDimensions.z;
   bytesPerSlice = pixelsPerSlice * bytesPerPixel;


   /*----- create buffer big enough for the volume -----*/
   tempdata = calloc(pixelsPerVolume, bytesPerPixel);
   if(tempdata == NULL){
      printf ("An error occurred allocating a volume buffer\n");
      return (-1);
   }

   /*----- read the voume data -----*/
   printf ("bytesPerPixel = %d\n", bytesPerPixel);
   printf ("pixelsPerVolume = %d\n", pixelsPerVolume);

   if ((l = read(input_fd, tempdata, (bytesPerPixel * pixelsPerVolume)))
      != (bytesPerPixel * pixelsPerVolume))
   {
      printf ("Reading volume failed\n");
      return (-1);
   }

   /* ---- Convert double input data to unsigned chars ---- */
   printf("Truncating input data from doubles to unsigned bytes...");
   _rawdata = (unsigned char*) calloc(pixelsPerVolume, sizeof(unsigned char));
   if(_rawdata == NULL){
      printf ("An error occurred allocating a volume buffer\n");
      return (-1);
   }
   for(l = 0; l < pixelsPerVolume; l++){
      _rawdata[l] = doubleToByte(tempdata[l]);
   }
   printf("done\n");

   /*----- close file and free buffer -----*/
   close(input_fd);
   free(tempdata);

   /* resample the dataset if it is too small */
   if(pixelsPerVolume < 1000000){
      printf("Resampling %d by %d by %d volume...", _VolumeDimensions.x, _VolumeDimensions.y, _VolumeDimensions.z);
      tempdata2 = malloc(pixelsPerVolume * 27 * sizeof(unsigned char));
      vpResample3D(_rawdata, _VolumeDimensions.x, _VolumeDimensions.y, _VolumeDimensions.z,
                   tempdata2, _VolumeDimensions.x * 3, _VolumeDimensions.y * 3, _VolumeDimensions.z * 3,
                   VP_UCHAR, VP_BSPLINE_FILTER);
      
      _VolumeDimensions.x *= 3;
      _VolumeDimensions.y *= 3;
      _VolumeDimensions.z *= 3;
      free(_rawdata);
      _rawdata = tempdata2;
      printf("done\n");
   }



   printf ("\nFile: [ %s ]\n", filename);
   printf ("Image Width  = %d pixels\n", _VolumeDimensions.x);
   printf ("Image Height = %d pixels\n", _VolumeDimensions.y);
   printf ("Image Frames = %d pixels\n", _VolumeDimensions.z);
   printf ("Pixel Depth = %d bits\n", _PixelDepth);
   if(_PixelFormat == C_INTENSITY8)
      printf ("Pixel format = C_INTENSITY8\n");
   else if(_PixelFormat == C_INTENSITY16)
      printf ("Pixel format = C_INTENSITY16\n");
   
   if(_PixelType == C_UNSIGNED_BYTE )
      printf ("Image type  = C_UNSIGNED_BYTE\n");
   else
      printf ("Image type  = C_UNSIGNED_SHORT\n");
   
   return 1;
}

int readVolume(char *filename){
   int     input_fd;
   
   /*----- open the volume file -----*/
   
   if ((input_fd = open(filename, O_RDONLY)) == -1){
      printf ("Can't open volume file: %s\n", filename);
      return (-1);
   }

   /*----- read header info from the volume file -----*/

   read(input_fd, &_MagicNumber, sizeof(int));
   printf("Magic Number is %x\n", _MagicNumber);
   
   if(_MagicNumber == 8524291)  return readED(input_fd, filename);
   else return readVol(input_fd, filename);

}

